api_key: {{ api_key }}
api_path: /api
databases:
  {% for db in databases %}
  {% for label, path in db.items() %}
  - label: {{ label }}
    uri: {{ path }}
    type: csv
  {% endfor %}
  {% endfor %}
encryption:
  enabled: false
  private_key: null
logging:
  backup_count: 5
  datefmt: '%Y-%m-%d %H:%M:%S'
  file: {{ logging["file"] }}
  format: '%(asctime)s - %(name)-14s - %(levelname)-8s - %(message)s'
  level: DEBUG
  max_size: 1024
  use_console: true
  loggers:
  - level: warning
    name: urllib3
  - level: warning
    name: requests
  - level: warning
    name: engineio.client
  - level: warning
    name: docker.utils.config
  - level: warning
    name: docker.auth
port: {{ port }}
server_url: {{ server_url }}
task_dir: {{ task_dir}}
{{- user_provided_config -}}
