allow_drop_all: false
api_path: /api
description: 'defaultserver'
ip: 0.0.0.0
logging:
  backup_count: 5
  datefmt: '%Y-%m-%d %H:%M:%S'
  file: test-server.log
  format: '%(asctime)s - %(name)-14s - %(levelname)-8s - %(message)s'
  level: DEBUG
  max_size: 1024
  use_console: true
  loggers:
  - level: warning
    name: urllib3
  - level: warning
    name: socketIO-client
  - level: warning
    name: socketio.server
  - level: warning
    name: engineio.server
  - level: warning
    name: sqlalchemy.engine
  - level: warning
    name: requests_oauthlib.oauth2_session
port: {{ port }}
uri: sqlite:///default.sqlite
jwt_secret_key: {{ jwt_secret_key }}
dev:
  host_uri: {{ host_uri }}
server_url: http://localhost:{{ port }}/api
ui:
  enabled: true
  port: {{ ui_port }}
{{ user_provided_config }}